"""Gen cmat for en/zh text."""
# pylint: disable=

from typing import List, Optional

import numpy as np
from sklearn.feature_extraction.text import TfidfVectorizer
from fastlid import fastlid
from logzero import logger
from fast_scores import fast_scores
from fast_scores.process_en import process_en
from fast_scores.en2zh import en2zh
from fast_scores.process_zh import process_zh

# fix on en/zh, warn if confidence too low
fastlid.set_languages = ["en", "zh"]


def gen_cmat(
    text1: List[str],
    text2: List[str],
    model: Optional[TfidfVectorizer] = None,
) -> np.ndarray:
    """Gen corr matrix for en/zh texts."""
    # logger.debug("enter gen_cmat")
    if isinstance(text1, str):
        text1 = [text1]
    if isinstance(text2, str):
        text1 = [text2]
    lang1, conf1 = fastlid("\n".join(text1))
    lang2, conf2 = fastlid("\n".join(text2))

    # ic(lang1)
    # ic(lang2)

    if conf1 < 0.1:
        logger.warning(" text1 dected as %s, but confidence too low: %s, make sure you supply english or chinese texts", lang1, conf1)

    if conf2 < 0.1:
        logger.warning(" text2 dected as %s, but confidence too low: %s, make sure you supply english or chinese texts", lang2, conf2)

    if lang1 in ["en"] and lang2 in ["en"]:
        logger.warning("Both texts are en...are you sure you supplied the correct files?")

    if lang1 in ["zh"] and lang2 in ["zh"]:
        logger.warning("Both texts are zh...are you you supplied the correct files?")

    logger.debug("gen text1a text2a")
    # _ = """
    if lang1 in ["en"] and lang2 in ["en"]:
        text1a = en2zh(process_en(text1))
        text2a = en2zh(process_en(text2))
    elif lang1 in ["en"] and lang2 in ["zh"]:
        text1a = en2zh(process_en(text1))
        text2a = process_zh(text2)
    elif lang1 in ["zh"] and lang2 in ["en"]:
        text1a = process_zh(text1)
        text2a = en2zh(process_en(text2))
    # if lang1 in ["zh"] and lang2 in ["zh"]:
    else:
        text1a = process_zh(text1)
        text2a = process_zh(text2)
    # """

    # text1a = globals()["process_" + lang1](text1)
    # text2a = globals()["process_" + lang2](text2)

    logger.debug("execute fast_scores")
    _ = fast_scores(text1a, text2a, model=model)

    return _


_ = """

cmat = gen_cmat(en, zh)
aset = cmat2aset(cmat)

i = -1

i += 1
i, en[aset[i][0]] if isinstance(aset[i][0], int) else "", zh[aset[i][1]] if isinstance
(
    aset[i][1], int
) else "", aset[i][2]

"""



"""Detect language via polyglot and fastlid."""
# pylint: disable=

from typing import Any, Callable, List, Optional

import polyglot.detect.base
from fastlid import fastlid
from logzero import logger
from polyglot.detect.base import UnknownLanguage
from polyglot.text import Detector

polyglot.detect.base.logger.setLevel("ERROR")


def with_func_attrs(**attrs: Any) -> Callable:
    """Define func_attrs."""

    def with_attrs(fct: Callable) -> Callable:
        for key, val in attrs.items():
            setattr(fct, key, val)
        return fct

    return with_attrs


# @with_func_attrs(set_languages=None)
# def detect(text: str) -> str:
def detect(text: str, set_languages: Optional[List[str]] = None) -> str:
    """Detect language via polyglot and fastlid.

    check first with fastlid, if conf < 0.3, check with polyglot.text.Detector

    Alternative in detec_alt.py
    """
    # if not text.strip(): return "en"
    fastlid.set_languages = set_languages
    lang, conf = fastlid(text)
    detect.lang_conf = lang, conf
    if conf >= 0.3 or lang in ["zh"]:
        return lang

    try:
        langs = [(elm.code[:2], elm.confidence) for elm in Detector(text).languages]
        detect.lang_conf = langs
        # lang, conf = _[0]
    except UnknownLanguage:
        if set_languages is None:
            def_lang = "en"
        else:
            # def_lang = set_languages[-1]
            def_lang = set_languages[0]
        logger.warning(
            " UnknownLanguage exception: probably snippet too short, setting to %s",
            def_lang,
        )
        langs = [(def_lang, 0)]
    except Exception as exc:
        logger.error(exc)
        langs = [("en", 0)]

    del conf

    # return first enrty's lang
    if set_languages is None:
        def_lang = langs[0][0]
    else:
        def_lang = "en"

        # pick the first in Detector(text).languages

        # just to silence pyright
        # set_languages_: List[str] = [""] if set_languages is None else set_languages

        for elm in langs:
            if elm[0] in set_languages:  # type: ignore
                def_lang = elm[0]
                break

    # set_languages is set
    if not isinstance(set_languages, (list, tuple)):
        logger.warning("set_languages (%s) ought to be a list/tuple")

    return def_lang



"""Map cell background color for pandas.DataFrame.

palette = sns.blend_palette(
    # ["pink", "palegreen", 'green'], N_COLORS).as_hex()
    # ["pink", "palegreen"], N_COLORS).as_hex()
    ["red", "palegreen"], N_COLORS).as_hex()
Refer to color_table_applymap.py
"""
# pylint: disable=invalid-name
palette = [
    # "#f00000",
    # "#f02315",
    "#e2482c",
    "#d36b41",
    "#c49057",
    "#b5b36c",
    "#a7d883",
    "#98fb98",
]
ncolors = len(palette)


def color_map(v, min_: float = 0, max_: float = 1):
    """Map cell background color.

    e.g. s_df = df.style.applymap(color_map, min_=min_, max_=max_, subset=["B"])
    or s_df = df.style.applymap(color_map, subset=['likelihood'])

    or
    s_df = df.style.applymap(color_map, subset=[2,])

    or
    s_df = df.style.applymap(color_map, subset=[df.columns[2])

    or
    s_df = df.style.applymap(color_map, subset=[*df.columns[1:3]] + [*df.columns[0:1]])
    """
    wd = (max_ - min_) / ncolors
    try:
        v = float(v)  # !!!
        pal = palette[min(ncolors - 1, int((v - min_) / wd))]
    except Exception:  # as e:  # wont style str etc.
        # logger.debug("%s", e)
        # return None
        return "wrap_text: true"

    return f"background-color: {pal}"



"""Save list np.ndarray pd.DataFrame as xlsx."""
# pylint: disable=abstract-class-instantiated
from pathlib import Path
from random import choices
from typing import List, Optional, Union

import numpy as np
import pandas as pd
from logzero import logger

from dzbee.color_map import color_map
from dzbee.gen_filename import gen_filename


def save_xlsx_tsv_csv(
    lst: Union[np.ndarray, List[Union[str, float]], pd.core.frame.DataFrame],
    file: Optional[Path] = None,
    # subset: List = [],
    file_ext: Optional[List[str]] = None,  # .tsv, .csv default .xlsx
) -> pd.core.frame.DataFrame:
    """Save list np.ndarray pd.DataFrame as xlsx.

    Args:
        lst: content to save to xlsx
        file: if None save to cwd with a random file name
        x subset: columns that are styled with color_apply
        file_ext: .xlsx or .csv or .tsv, default: .xlsx
    Returns:
        pd.DataFrame
    """
    if file is None:
        _ = "".join([*"aa"] + choices("abcdefg", k=7) + choices("12345", k=3))
        file = Path(_)
    if file_ext is None:
        file_ext = [".xlsx"]
    columns = [str(elm) for elm in list(range(np.array(lst).shape[1]))]

    try:
        df_lst = pd.DataFrame(np.array(lst), columns=columns, dtype="object")
    except Exception as exc:
        logger.error(exc)
        raise

    # return styled df_lst if no filename provided
    # if file is None: return df_lst

    _ = """
    if Path(file).suffix not in [".xlsx"]:
        file = f"{file}.xlsx"
    if Path(file).suffix not in [f"{file_ext}"]:
        file = f"{file}{file_ext}"
    # """

    # save .csv
    # if file_ext in [".csv"]:
    if ".csv" in file_ext:
        _ = file.with_suffix(".csv")
        _ = gen_filename(_)
        df_lst.to_csv(_, index=False, header=False, encoding="gbk")  # or GB18030
        logger.info("csv written to %s", Path(_).absolute())

    # save .tsv
    # if file_ext in [".tsv"]:
    if ".tsv" in file_ext:
        _ = file.with_suffix(".tsv")
        _ = gen_filename(_)
        df_lst.to_csv(_, sep="\t", index=False, header=False)
        logger.info("tsv written to %s", Path(_).absolute())

    if ".xlsx" not in file_ext:
        return df_lst

    # proceed to process .xlsx
    # color the likelihood column (3rd col)

    # s_df = df_lst.style.applymap(color_map, subset=[*df_lst.columns[2:3]])

    _ = file.with_suffix(".xlsx")
    _ = Path(gen_filename(_))

    logger.debug("filep: %s, \n\t%s", _, df_lst.head(1))

    subset = list(df_lst.columns[2:3])  # 3rd col
    s_df = df_lst.style.applymap(color_map, subset=subset)
    try:
        # writer = pd.ExcelWriter(_)
        with pd.ExcelWriter(_, engine="xlsxwriter") as writer:
            s_df.to_excel(writer, index=False, header=False, sheet_name="Sheet1")
            writer.sheets["Sheet1"].set_column("A:A", 70)
            writer.sheets["Sheet1"].set_column("B:B", 70)

        # writer.close()
        logger.info("xlsx written to \n\t %s", Path(_).absolute())
    except Exception as exc:
        logger.exception("Unable to save: %s", exc)
        raise

    return df_lst


def test_save_xlsx1():
    """Test save_xlsx Path."""
    filepath = Path("data/test.xlsx")
    lst = [[0, 1, "a"], [1, 1, 1]]
    res = save_xlsx_tsv_csv(lst, filepath)
    del res
    assert Path(filepath).exists()



"""Plot cmat/aset to html via holoviews and plotly.

aset = cmat2aset(cmat.T)
arr = np.array(aset)
# arr[arr == ""] = np.nan
arr[arr == ""] = 0
arr = arr.astype(float)

# set value to np.nan for ""
_ = [elm.tolist() if elm[2] != 0 else elm.tolist()[:2] + [np.nan] for elm in arr]


# remove rows containing nan
arr = arr[~np.isnan(arr).any(axis=1)]
_ = arr.copy()

# remove rows with first entry being nan
arr = arr[~np.isnan(arr[:, 0])]
_ = arr.copy()

# remove rows with first/second entry being nan
arr = arr[~(np.isnan(arr[:, 0]) | np.isnan(arr[:, 1]))]
_ = arr.copy()

tfv = np.isnan(arr)
arr[np.logical_not(tfv)]

"""
# pylint: disable=invalid-name

import os
from pathlib import Path
from typing import List, Optional

# from icecream import ic
# from holoviews import opts
import holoviews as hv
import joblib
import numpy as np
from cmat2aset import cmat2aset  # pylint: disable=import-error  # likely caused by pyc
from logzero import logger

# hv.extension('bokeh')
hv.extension("plotly")


def cmat2html(
    # data: np.ndarray,
    cmat: np.ndarray,
    aset: Optional[List] = None,
    xlabel: str = "x",
    ylabel: str = "y",
    # startfile: bool = True,
    show_plot: bool = True,
):
    """Plot cmat/aset to html via holoviews and plotly.

    Args:
        cmat: correlation matrix
        xlabel: x-axis label in the plots
        ylabel: y-axis label in the plots
        aset: align set
        show_plot: try to show html in a browser if set (default True) via os.startfile

    e.g.
    cmat = gen_cmat(en, zh)
    data2html(cmat, xlable="zh (x)", ylabel="en (y)")

    aset = cmat2aset(cmat)
    # (32, 35, 0.5)
    i, j, llh = aset[-1]
    assert round(cmat[j, i], 2) == llh
    en[i]. zh[j], llh
    # ('Homepage', '返回首页', 0.5)

    # (26, 28, 0.48)
    i, j, llh = aset[-8]
    assert round(cmat[j, i], 2) == llh
    en[i], zh[j], llh
    # ("`Were you asked to tea?' she demanded, tying an apron over her neat black frock, and standing with a spoonful of the leaf poised over the pot.",
    # '“是请你来吃茶的吗？”她问，把一条围裙系在她那干净的黑衣服上，就这样站着，拿一匙茶叶正要往茶壶里倒。',
    # 0.48)

    """
    # data changed to cmat
    # ic(data.shape)

    # _ = [[*elm] + [data[elm]] for elm in np.ndindex(data.shape)]

    # _ = [[*elm] + [v] for elm, v in np.ndenumerate(data)]
    _ = [[*elm, v] for elm, v in np.ndenumerate(cmat)]
    hm1 = hv.HeatMap(
        _,
        label="(z) likelihood heatmap",
        name="name1",
    )

    hm1.opts(
        # xticks=None,
        # tools=['hover'],  # only for bokeh
        # colorbar=True,
        # cmap="viridis_r",
        xlabel=xlabel,
        ylabel=ylabel,
        # cmap="viridis_r",
        # cmap="viridis",
        cmap="gist_earth_r",
        # cmap="summer_r",
        # cmap="fire_r",
    )

    # aset
    # aset = cmat2aset(cmat.T)
    # cmat
    if aset is None:
        # aset = cmat2aset(cmat.T)
        aset = cmat2aset(cmat)

    arr = np.array(aset, dtype=object)

    # convert "" in col3 to nan
    arr[:, 2][arr[:, 2] == ""] = np.nan
    arr[arr == ""] = 0
    arr = arr.astype(float)

    # set value to np.nan for "": old way
    # _ = [elm.tolist() if elm[2] != 0 else elm.tolist()[:2]
    # + [np.nan] for elm in arr]

    hm2 = hv.HeatMap(
        arr,
        label="(z) likelihood align trace",
    )

    hm2.opts(
        # xticks=None,
        # tools=['hover'],  # only for bokeh
        colorbar=True,
        # cmap="viridis_r",
        xlabel=xlabel,
        ylabel=ylabel,
        # cmap="viridis_r",
        # cmap="viridis",
        cmap="gist_earth_r",
        # cmap="summer_r",
        # cmap="fire_r",
        # labelled=["en", "zh", "llh"]  # ["x", "y", "z"]
    )
    # hv.save(hm, 'heatmap.html')
    # os.startfile("heatmap.html")

    hv.save(hm1 + hm2, "heatmap12.html")
    # hv.save(hm1 * hm, "heatmap1x2.html")

    logger.info(" html file written to %s", Path("heatmap12.html").resolve())

    # if startfile:
    if show_plot:
        try:
            os.startfile("heatmap12.html")
            logger.info(" Showing heatmap12.html in the default browser.")
        except Exception as e:
            logger.error(e)


if __name__ == "__main__":
    # 36x33
    _ = joblib.load(r"data\cmat_36x33(zh-en).lzma")
    cmat2html(_)



"""Gen aligned pairs for given list1 list2 and aset."""
# pylint: disable=invalid-name
from typing import List
from logzero import logger


def gen_pairs(list1: List[str], list2: List[str], aset: List[str]) -> List[str]:
    """Gen aligned pairs for given list1 list2 and aset."""
    res = []
    for elm in aset:
        try:
            ix, iy, val, *_ = elm
        except Exception as e:
            logger.exception(e)
            raise

        term = []
        try:
            iy = int(iy)  # note iy, not ix
            _ = list1[iy]
        except Exception:
            _ = ""
        term.append(_)

        try:
            ix = int(ix)
            _ = list2[ix]
        except Exception:
            _ = ""
        term.append(_)

        try:
            val = float(val)
        except Exception:
            val = ""
        term.append(val)

        res.append(term)

    return res



"""Generate a feasible filename for a given file name by adding extra number."""
from pathlib import Path
from random import sample
from string import ascii_lowercase


def gen_filename(filename, sep="_", rand=False):
    """Generate a feasible filename for a given file name by adding extra number.

    >>> gen_filename('gen_filename.py')
    'gen_filename_01.py'
    >>> len(gen_filename('gen_filename.py', rand=True))
    23
    """
    # filename = str(filename)

    # file_path = Path(filename).absolute()
    file_path = Path(filename)
    if not file_path.exists():
        return filename

    parent = file_path.parent
    stem = file_path.stem
    suffix = file_path.suffix
    elm = 0
    while elm < 99 and not rand:
        elm += 1
        extra = f"{elm:02d}"
        f_name = f"{stem}{sep}{extra}{suffix}"
        if not (parent / f_name).exists():
            break
    else:
        extra = "".join(sample(ascii_lowercase, 7))
    f_name = f"{stem}{sep}{extra}{suffix}"

    return (parent / f_name).as_posix()



"""Get the common prefix."""
from typing import List
from pathlib import Path
from itertools import takewhile


def common_prefix(strings: List[str]) -> str:
    """Get common prefix.

    >>> str_ = ['my_prefix_what_ever', 'my_prefix_what_so_ever', 'my_prefix_doesnt_matter']  # NOQA
    >>> common_prefix(str_)
    'my_prefix_'
    """
    if isinstance(strings, str):
        strings = strings.split()

    if not isinstance(strings, list):
        strings = list(strings)

    # if just one item, retrun itself, without the extra "_"
    if len(strings) == 1:
        elm = strings[0]
        # remove suffix if any
        elm = Path(elm)
        return str(elm).strip(elm.suffix)

    strings = [str(_) for _ in strings if _]
    c_prefix = "".join(
        c[0] for c in takewhile(lambda x: all(x[0] == y for y in x), zip(*strings))
    )
    if c_prefix:
        return c_prefix.strip("_") + "_"  # NOQA

    # if emptry c_prefix
    suff = Path(strings[0]).suffix
    return strings[0].strip(suff).strip("_") + "_"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dzbee']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=3.1.2,<4.0.0',
 'Morfessor>=2.0.6,<3.0.0',
 'XlsxWriter>=3.0.3,<4.0.0',
 'aset2pairs>=0.1.0,<0.2.0',
 'cchardet>=2.1.7,<3.0.0',
 'icecream>=2.1.1,<3.0.0',
 'install>=1.3.5,<2.0.0',
 'json-de2zh==0.1.1-alpha.0',
 'logzero>=1.7.0,<2.0.0',
 'polyglot>=16.7.4,<17.0.0',
 'typer>=0.4.1,<0.5.0']

entry_points = \
{'console_scripts': ['dzbee = dzbee.__main__:app']}

setup_kwargs = {
    'name': 'dzbee',
    'version': '0.1.1a1',
    'description': 'pack_name descr ',
    'long_description': '# dzbee\n[![pytest](https://github.com/ffreemt/dzbee/actions/workflows/routine-tests.yml/badge.svg)](https://github.com/ffreemt/dzbee/actions)[![python](https://img.shields.io/static/v1?label=python+&message=3.8%2B&color=blue)](https://www.python.org/downloads/)[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)[![PyPI version](https://badge.fury.io/py/dzbee.svg)](https://badge.fury.io/py/dzbee)\n\nAlign german(de)-chinese(zh) texts, fast\n\n### Python 3.8 Only\n\n## Pre-Install `fasttext`, `pycld2`, `PyICU`\n*   If your computer **does not** have a C++ compiler,\n search for needed wheels at  https://www.lfd.uci.edu/~gohlke/pythonlibs/ and install, e.g.,\n    ```\n     pip install fasttext-0.9.2-cp38-cp38-win_amd64.whl\n     pip install pycld2-0.41-cp38-cp38-win_amd64.whl\n     pip install PyICU-2.8.1-cp38-cp38-win_amd64.whl\n    ```\n*   If your computer *does* have a C++ compiler\n    ```\n       pip insall fasttext pycld2 PyICU\n       # poetry add fasttext pycld2 PyICU\n    ```\n\n## Install it\n\n```shell\npip install dzbee\n\n# poetry add dzbee\n# pip install git+https://github.com/ffreemt/dzbee\n# poetry add git+https://github.com/ffreemt/dzbee\n# git clone https://github.com/ffreemt/dzbee && cd dzbee\n```\n\n## Use it\n```bash\ndzbee file1 file2\n\n```\n',
    'author': 'ffreemt',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ffreemt/dzbee',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.3,<4.0.0',
}


setup(**setup_kwargs)
